@@ -8,10 +8,11 @@ from django.http import HttpResponseRedirect
 from django.shortcuts import redirect
 from django.urls import reverse
 
-from db.base.models import Artifact, DemodData, ExportedFrameset, Mode, \
-    Operator, Satellite, Telemetry, Tle, Transmitter, TransmitterEntry, \
-    TransmitterSuggestion
-from db.base.tasks import check_celery, decode_all_data
+from db.base.models import Artifact, DemodData, ExportedFrameset, \
+    LatestTleSet, Mode, Operator, Satellite, Telemetry, Tle, Transmitter, \
+    TransmitterEntry, TransmitterSuggestion
+from db.base.tasks import check_celery, decode_all_data, update_tle_sets
+from db.base.utils import update_latest_tle_sets
 
 
 @admin.register(Mode)
@@ -30,8 +31,8 @@ class OperatorAdmin(admin.ModelAdmin):
 @admin.register(Satellite)
 class SatelliteAdmin(admin.ModelAdmin):
     """Defines Satellite view in django admin UI"""
-    list_display = ('id', 'name', 'norad_cat_id', 'status', 'decayed')
-    search_fields = ('name', 'norad_cat_id')
+    list_display = ('id', 'name', 'norad_cat_id', 'status', 'decayed', 'norad_follow_id')
+    search_fields = ('name', 'norad_cat_id', 'norad_follow_id')
     list_filter = ('status', 'decayed')
 
     def get_urls(self):
@@ -229,6 +230,48 @@ class TleAdmin(admin.ModelAdmin):
         """Return the satellite name"""
         return obj.satellite.name
 
+    def get_urls(self):
+        """Returns django urls for Tle view
+
+        update_tle_sets -- url for the update_tle_sets function
+
+        :returns: Django urls for the Tle admin view
+        """
+        urls = super(TleAdmin, self).get_urls()
+        my_urls = [
+            url(r'^update_tle_sets/$', self.update_tle_sets, name='update_tle_sets'),
+        ]
+        return my_urls + urls
+
+    def update_tle_sets(self, request):  # pylint: disable=R0201
+        """Returns the admin home page, while triggering a Celery update tle sets task
+
+        :returns: Admin home page
+        """
+        update_tle_sets.delay()
+        messages.success(request, 'Update TLE sets task was triggered successfully!')
+        return redirect(reverse('admin:index'))
+
+    def save_model(self, request, obj, form, change):
+        super().save_model(request, obj, form, change)
+        update_latest_tle_sets(satellites=[obj.satellite.pk])
+
+    def delete_model(self, request, obj):
+        super().delete_model(request, obj)
+        update_latest_tle_sets(satellites=[obj.satellite.pk])
+
+    def delete_queryset(self, request, queryset):
+        satellites = [tle.satellite.pk for tle in queryset]
+        super().delete_queryset(request, queryset)
+        update_latest_tle_sets(satellites=satellites)
+
+
+@admin.register(LatestTleSet)
+class LatestTleSetAdmin(admin.ModelAdmin):
+    """Defines LatestTleSet view in django admin UI"""
+    list_display = ('satellite', 'latest', 'latest_distributable', 'last_modified')
+    list_filter = ('satellite__name', )
+
 
 @admin.register(Telemetry)
 class TelemetryAdmin(admin.ModelAdmin):
