@@ -1,6 +1,5 @@
 """SatNOGS DB Celery task functions"""
 import csv
-import json
 import logging
 import tempfile
 from datetime import datetime, timedelta
@@ -16,12 +15,13 @@ from django.core.validators import URLValidator
 from django.template.loader import render_to_string
 from django.urls import reverse
 from django.utils.timezone import make_aware
-from satellite_tle import fetch_all_tles, fetch_tle_from_celestrak, fetch_tles
+from satellite_tle import fetch_all_tles, fetch_tle_from_celestrak
 from sgp4.earth_gravity import wgs72
 from sgp4.io import twoline2rv
 
 from db.base.models import DemodData, ExportedFrameset, Satellite, Tle
-from db.base.utils import cache_statistics, decode_data
+from db.base.utils import cache_statistics, decode_data, get_tle_sources, \
+    update_latest_tle_sets
 
 LOGGER = logging.getLogger('db')
 
@@ -33,7 +33,7 @@ def check_celery():
 
 
 @shared_task
-def update_satellite(norad_id, update_name=True, update_tle=True):
+def update_satellite(norad_id, update_name=True):
     """Task to update the name and/or the tle of a satellite, or create a
        new satellite in the db if no satellite with given norad_id can be found"""
 
@@ -49,11 +49,6 @@ def update_satellite(norad_id, update_name=True, update_tle=True):
     if update_name:
         satellite.name = tle[0]
 
-    if update_tle:
-        satellite.tle_source = 'Celestrak (satcat)'
-        satellite.tle1 = tle[1]
-        satellite.tle2 = tle[2]
-
     satellite.save()
 
     if satellite_created:
@@ -62,104 +57,75 @@ def update_satellite(norad_id, update_name=True, update_tle=True):
         print('Updated satellite {}: {}'.format(satellite.norad_cat_id, satellite.name))
 
 
-@shared_task
-def update_all_tle():
-    """Task to update all satellite TLEs"""
-
-    satellites = Satellite.objects.exclude(status__exact='re-entered')
-    norad_ids = set(int(sat.norad_cat_id) for sat in satellites)
-
-    # Filter only officially announced NORAD IDs
-    temporary_norad_ids = {norad_id for norad_id in norad_ids if norad_id >= 99000}
-    public_norad_ids = norad_ids - temporary_norad_ids
-
-    tles = fetch_tles(public_norad_ids)
-
-    missing_norad_ids = []
-    for satellite in satellites:
-        norad_id = satellite.norad_cat_id
-
-        if norad_id not in list(tles.keys()):
-            # No TLE available for this satellite
-            missing_norad_ids.append(norad_id)
-            continue
-
-        source, tle = tles[norad_id]
-
-        if satellite.tle1 and satellite.tle2:
-            try:
-                current_sat = twoline2rv(satellite.tle1, satellite.tle2, wgs72)
-                new_sat = twoline2rv(tle[1], tle[2], wgs72)
-                if new_sat.epoch < current_sat.epoch:
-                    # Epoch of new TLE is larger then the TLE already in the db
-                    continue
-            except ValueError:
-                LOGGER.error('ERROR: TLE malformed for %s', norad_id)
-                continue
-
-        satellite.tle_source = source
-        satellite.tle1 = tle[1]
-        satellite.tle2 = tle[2]
-        satellite.save()
-
-        print('Updated TLE for {}: {} from {}'.format(norad_id, satellite.name, source))
-
-    for norad_id in sorted(missing_norad_ids):
-        satellite = satellites.get(norad_cat_id=norad_id)
-        print('NO TLE found for {}: {}'.format(norad_id, satellite.name))
-
-    for norad_id in sorted(temporary_norad_ids):
-        satellite = satellites.get(norad_cat_id=norad_id)
-        print('Ignored {} with temporary NORAD ID {}'.format(satellite.name, norad_id))
-
-
 @shared_task
 def update_tle_sets():
     """Task to update all satellite TLEs"""
-    satellites = Satellite.objects.exclude(status='re-entered')
-    norad_ids = set(int(sat.norad_cat_id) for sat in satellites)
+    satellites = Satellite.objects.exclude(status='re-entered').exclude(status='future')
+    catalog_norad_ids = set()
+    for satellite in satellites:
+        # Check if satellite follows a NORAD ID and it is officially announced
+        if satellite.norad_follow_id and satellite.norad_follow_id < 99000:
+            catalog_norad_ids.add(satellite.norad_follow_id)
+        # Check if satellite has a NORAD ID that is officially announced
+        elif satellite.norad_cat_id < 99000:
+            catalog_norad_ids.add(satellite.norad_cat_id)
 
-    # Filter only officially announced NORAD IDs
-    catalog_norad_ids = {norad_id for norad_id in norad_ids if norad_id < 99000}
-
-    # Check for TLE custom sources
-    other_sources = {}
-    if settings.TLE_SOURCES_JSON:
-        try:
-            sources_json = json.loads(settings.TLE_SOURCES_JSON)
-            other_sources['sources'] = list(sources_json.items())
-        except json.JSONDecodeError as error:
-            print('TLE Sources JSON ignored as it is invalid: {}'.format(error))
-    if settings.SPACE_TRACK_USERNAME and settings.SPACE_TRACK_PASSWORD:
-        other_sources['spacetrack_config'] = {
-            'identity': settings.SPACE_TRACK_USERNAME,
-            'password': settings.SPACE_TRACK_PASSWORD
-        }
+    other_sources = get_tle_sources()
 
     print("==Fetching TLEs==")
     tles = fetch_all_tles(catalog_norad_ids, **other_sources)
 
     for satellite in satellites:
         norad_id = satellite.norad_cat_id
+        if satellite.norad_follow_id:
+            norad_id = satellite.norad_follow_id
+
         if norad_id in tles.keys():
             for source, tle in tles[norad_id]:
-                (tle, created) = Tle.objects.get_or_create(
+                try:
+                    sgp4_tle = twoline2rv(tle[1], tle[2], wgs72)
+                except ValueError:
+                    print(
+                        '[ERROR] {} - {} - {}: TLE malformed'.format(
+                            satellite.name, norad_id, source
+                        )
+                    )
+                    continue
+
+                try:
+                    last_tle_from_source = Tle.objects.filter(
+                        satellite=satellite, tle_source=source
+                    ).latest('updated')
+                    sgp4_last_tle = twoline2rv(
+                        last_tle_from_source.tle1, last_tle_from_source.tle2, wgs72
+                    )
+
+                    if (sgp4_tle.epoch == sgp4_last_tle.epoch
+                            and last_tle_from_source.tle0 == tle[0]  # noqa: W503
+                            and last_tle_from_source.tle1 == tle[1]  # noqa: W503
+                            and last_tle_from_source.tle2 == tle[2]):  # noqa: W503
+                        print(
+                            '[EXISTS] {} - {} - {}: TLE set already exists'.format(
+                                satellite.name, norad_id, source
+                            )
+                        )
+                        continue
+
+                except Tle.DoesNotExist:
+                    pass
+
+                Tle.objects.create(
                     tle0=tle[0], tle1=tle[1], tle2=tle[2], satellite=satellite, tle_source=source
                 )
-                if created:
-                    print(
-                        '{} - {} - {}: [ADDED] TLE set is added'.format(
-                            satellite.name, norad_id, source
-                        )
-                    )
-                else:
-                    print(
-                        '{} - {} - {}: [EXISTS] TLE set already exists'.format(
-                            satellite.name, norad_id, source
-                        )
+
+                print(
+                    '[ADDED] {} - {} - {}: TLE set is added'.format(
+                        satellite.name, norad_id, source
                     )
+                )
         else:
-            print('{} - {}: [NOT FOUND] TLE set wasn\'t found'.format(satellite.name, norad_id))
+            print('[NOT FOUND] {} - {}: TLE set wasn\'t found'.format(satellite.name, norad_id))
+    update_latest_tle_sets()
 
 
 @shared_task
@@ -286,3 +252,9 @@ def decode_recent_data():
             # an object could have failed decoding for a number of reasons,
             # keep going
             continue
+
+
+@shared_task
+def decode_current_frame(norad, handle):
+    """Task to trigger a decode of a current frame for a satellite."""
+    decode_data(norad, period=None, handle=handle)
